@@ -64,6 +64,7 @@ enum GGUFArch
     ARCH_GLM4 = 9,
     ARCH_GEMMA3N = 10,
     ARCH_GPTOSS = 11,
+    ARCH_DEEPSEEK2 = 12,
 };
 
 struct FileFormatExtraMeta
@@ -107,6 +108,7 @@ const std::vector<TopPicksData> gpttype_get_top_picks_data();
 
 bool sdtype_load_model(const sd_load_model_inputs inputs);
 sd_generation_outputs sdtype_generate(const sd_generation_inputs inputs);
+sd_generation_outputs sdtype_upscale(const sd_upscale_inputs inputs);
 sd_info_outputs sdtype_get_info();
 
 bool whispertype_load_model(const whisper_load_model_inputs inputs);
