@@ -787,6 +787,7 @@ sd_generation_outputs sdtype_generate(const sd_generation_inputs inputs)
     bool is_wan = (loadedsdver == SDVersion::VERSION_WAN2 || loadedsdver == SDVersion::VERSION_WAN2_2_I2V || loadedsdver == SDVersion::VERSION_WAN2_2_TI2V);
     bool is_qwenimg = (loadedsdver == SDVersion::VERSION_QWEN_IMAGE);
     bool is_kontext = (loadedsdver==SDVersion::VERSION_FLUX && !loaded_model_is_chroma(sd_ctx));
+    bool is_flux2 = (loadedsdver == SDVersion::VERSION_FLUX2 || loadedsdver == SDVersion::VERSION_FLUX2_KLEIN);
 
     if (loadedsdver == SDVersion::VERSION_FLUX)
     {
@@ -808,12 +809,12 @@ sd_generation_outputs sdtype_generate(const sd_generation_inputs inputs)
 
     if(!remove_limits && loadedsdver == SDVersion::VERSION_Z_IMAGE)
     {
-        if(sd_params->cfg_scale > 3.0f)
+        if(sd_params->cfg_scale > 4.0f)
         {
             if (!sd_is_quiet && sddebugmode) {
-                printf("Z-Image: clamping CFG Scale to 3.0 to preserve quality\n");
+                printf("Z-Image: clamping CFG Scale to 4.0 to preserve quality\n");
             }
-            sd_params->cfg_scale = 3.0f;
+            sd_params->cfg_scale = 4.0f;
         }
     }
 
@@ -914,7 +915,7 @@ sd_generation_outputs sdtype_generate(const sd_generation_inputs inputs)
                     wan_imgs.push_back(extraimage_reference);
                 }
             }
-            else if(is_qwenimg)
+            else if(is_qwenimg || is_flux2)
             {
                 uint8_t * loaded = load_image_from_b64(extra_image_data[i],nx2,ny2);
                 if(loaded)
